package com.urovo.wugumofang.view

import android.R
import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.Rect
import android.graphics.drawable.Drawable
import android.view.View
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import androidx.recyclerview.widget.RecyclerView.ItemDecoration

/**
 * @author Sasuke on 2018/7/11.
 */
/**
 * 万能分割线
 * Created by ChenSS on 2016/9/21.
 */
class DividerUniteItemDecoration : ItemDecoration {
    private var mPaint: Paint? = null

    //取名mDivider似乎更恰当
    private var mDrawable: Drawable? = null

    //分割线高度，默认为1px
    private var mDividerHeight = 2

    //列表的方向
    private var mOrientation = 0

    companion object {
        //系统自带的参数
        private val ATTRS = intArrayOf(R.attr.listDivider)

        //水平
        const val HORIZONTAL_LIST = RecyclerView.HORIZONTAL

        //垂直
        const val VERTICAL_LIST = RecyclerView.VERTICAL
    }

    /**
     * 默认分割线：高度为2px，颜色为灰色
     *
     * @param context     上下文
     * @param orientation 列表方向
     */
    constructor(context: Context, orientation: Int) {
        setOrientation(orientation)
        val a = context.obtainStyledAttributes(ATTRS)
        mDrawable = a.getDrawable(0)
        a.recycle()
    }

    /**
     * 自定义分割线
     *
     * @param context     上下文
     * @param orientation 列表方向
     * @param drawableId  分割线图片
     */
    constructor(context: Context?, orientation: Int, drawableId: Int) {
        setOrientation(orientation)
        mDrawable = ContextCompat.getDrawable(context!!, drawableId)
        mDividerHeight = mDrawable!!.intrinsicHeight
    }

    /**
     * 自定义分割线
     *
     * @param context       上下文
     * @param orientation   列表方向
     * @param dividerHeight 分割线高度
     * @param dividerColor  分割线颜色
     */
    constructor(
        context: Context?, orientation: Int,
        dividerHeight: Int, dividerColor: Int
    ) {
        setOrientation(orientation)
        mDividerHeight = dividerHeight
        //抗锯齿画笔
        mPaint = Paint(Paint.ANTI_ALIAS_FLAG)
        mPaint!!.color = dividerColor
        //填满颜色
        mPaint!!.style = Paint.Style.FILL
    }

    /**
     * 设置方向
     *
     * @param orientation
     */
    fun setOrientation(orientation: Int) {
        require(!(orientation < 0 || orientation > 2)) { "invalid orientation" }
        mOrientation = orientation
    }

    /**
     * 绘制分割线之后,需要留出一个外边框,就是说item之间的间距要换一下
     *
     * @param outRect outRect.set(0, 0, 0, 0);的四个参数理解成margin就好了
     * @param view    视图
     * @param parent  父级view
     * @param state
     */
    override fun getItemOffsets(
        outRect: Rect,
        view: View,
        parent: RecyclerView,
        state: RecyclerView.State
    ) {
        super.getItemOffsets(outRect, view, parent, state)
        //获取layoutParams参数
        val layoutParams = view.layoutParams as RecyclerView.LayoutParams
        //当前位置
        val itemPosition = layoutParams.viewLayoutPosition
        //ItemView数量
        val childCount = parent.adapter!!.itemCount
        when (mOrientation) {
            VERTICAL_LIST -> {
                if (itemPosition == 0) {
                    outRect.set(mDividerHeight, 0, mDividerHeight, 0)
                } else {
                    outRect.set(0, 0, mDividerHeight, 0)
                }
            }
            HORIZONTAL_LIST -> {
                if (itemPosition == 0) {
                    outRect.set(0, mDividerHeight, 0, mDividerHeight)
                } else {
                    outRect.set(0, 0, 0, mDividerHeight)
                }
            }
        }
    }

    /**
     * 绘制分割线
     *
     * @param c Canvas
     * @param parent RecyclerView
     * @param state
     */
    override fun onDraw(c: Canvas, parent: RecyclerView, state: RecyclerView.State) {
        super.onDraw(c, parent, state)
        if (mOrientation == HORIZONTAL_LIST) {
            drawHorizontal(c, parent)
        } else {
            drawVertical(c, parent)
        }
    }

    /**
     * 绘制横向 item 分割线
     *
     * @param canvas 画布
     * @param parent 父容器
     */
    private fun drawHorizontal(canvas: Canvas, parent: RecyclerView) {
        val x = parent.paddingLeft
        val width = parent.measuredWidth - parent.paddingRight
        //getChildCount()(ViewGroup.getChildCount) 返回的是显示层面上的“所包含的子 View 个数”。
        val childSize = parent.childCount
        for (i in 0 until childSize) {
            val child = parent.getChildAt(i)
            val layoutParams = child.layoutParams as RecyclerView.LayoutParams
            //item底部的Y轴坐标+margin值
            val y = child.bottom + layoutParams.bottomMargin
            val height = y + mDividerHeight
            if (mDrawable != null) {
                //setBounds(x,y,width,height); x:组件在容器X轴上的起点 y:组件在容器Y轴上的起点
                // width:组件的长度 height:组件的高度
                mDrawable!!.setBounds(x, y, width, height)
                mDrawable!!.draw(canvas)
            }
            if (mPaint != null) {
                canvas.drawRect(x.toFloat(), y.toFloat(), width.toFloat(), height.toFloat(), mPaint)
            }
        }
    }

    /**
     * 绘制纵向 item 分割线
     *
     * @param canvas
     * @param parent
     */
    private fun drawVertical(canvas: Canvas, parent: RecyclerView) {
        val top = parent.paddingTop
        val bottom = parent.measuredHeight - parent.paddingBottom
        val childSize = parent.childCount
        for (i in 0 until childSize) {
            val child = parent.getChildAt(i)
            val layoutParams = child.layoutParams as RecyclerView.LayoutParams
            val left = child.right + layoutParams.rightMargin
            val right = left + mDividerHeight
            if (mDrawable != null) {
                mDrawable!!.setBounds(left, top, right, bottom)
                mDrawable!!.draw(canvas)
            }
            if (mPaint != null) {
                canvas.drawRect(
                    left.toFloat(),
                    top.toFloat(),
                    right.toFloat(),
                    bottom.toFloat(),
                    mPaint
                )
            }
        }
    }
}