package com.urovo.activity.attendancemanagement.signinandout.map

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.util.Log
import android.view.View
import com.amap.api.location.AMapLocation
import com.amap.api.location.AMapLocationClient
import com.amap.api.location.AMapLocationClientOption
import com.amap.api.maps2d.*
import com.amap.api.maps2d.model.BitmapDescriptorFactory
import com.amap.api.maps2d.model.LatLng
import com.amap.api.maps2d.model.MyLocationStyle
import com.urovo.wugumofang.R

/**
 * @author Sasuke on 2019-01-06.
 * 地图支配者（对高德地图的封装）
 *
 * 使用方法：
 * ① 当前Activity 实现 LocationSource 接口
 * ② 声明 var mapDominator: MapDominator? = null
 * ③ 【这个只是用来实例化mapDominator，并没有初始化地图】
 * mapDominator = MapDominator.MapBuilder(instanceState, this, mapView).build()
 *
 *
 * ④ mapDominator?.start()?.setLocateButton(locateBtn)
 * 【一定要跟上面分开，要先build()，再执行 start()，再执行后面的挂按钮监听之类的其它杂项配置。
 *      这个才是真正的初始化地图参数，因为里面的 setLocationSource()方法会导致跳转到 activate()方法，激活地图，
 *      而如果把build()放在最后做，那此时activate()方法里的mapDominator为空，就不会执行挂监听器的操作了】
 *
 *  ⑤  【重写 activity 重载的 activate()激活方法，实现定位监听的回调后的操作】
 *  override fun activate(listener: LocationSource.OnLocationChangedListener?) {
                mapDominator?.activateMap(listener, successCallback = { location ->
                        showMsg("成功获取地理信息， 位置在: \n ${location.address}")
                }, failedCallback = { location ->
                        AlertDialog.Builder(this@SignInAndOutActivity)
                        .setMessage("获取地理信息失败，请检查设置定位权限"")
                        .setNegativeButton("算了", null)
                        .setPositiveButton("去开启权限", { dialogInterface, i -> startActivity(Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)) })//跳转系统定位设置
                        .show()
                        Log.e("AmapErr", "高德地图获取地理信息失败：${location.getErrorCode()} : ${location.getErrorInfo()}")
                })
                }

 *  ⑥ 在当前 activitiy 重载的各个方法里添加以下操作：
 *              mapDominator?.deactivate()
 *              mapDominator?.onDestroy() //在activity执行onDestroy时执行mMapView.onDestroy()，销毁地图
 *              mapDominator?.onResume() //在activity执行onResume时执行mMapView.onResume ()，重新绘制加载地图
 *              mapDominator?.onPause() //在activity执行onPause时执行mMapView.onPause ()，暂停地图的绘制
 *              mapDominator?.onSaveInstanceState(outState)  //在activity执行onSaveInstanceState时执行mMapView.onSaveInstanceState (outState)，保存地图当前的状态


 */
class MapDominator(var builder: MapBuilder?) {

    /**
     * 取消激活
     */
    fun deactivate() {
        builder?.run {
            mListener = null
            mlocationClient?.run {
                stopLocation()
                onDestroy()
            }
            mlocationClient = null
        }
    }

    /**
     * 手动定位
     */
    fun startLocate(){
        builder?.mlocationClient?.startLocation()
    }

    /**
     * 放在Activity的onDestroy()方法里
     */
    fun onDestroy() {
        builder?.run {
            mapView?.onDestroy()
            mlocationClient?.let { it.onDestroy() }
        }
    }

    /**
     * 放在Activity的 onResume()方法里
     */
    fun onResume() {
        builder?.run {
            mapView?.onResume()
        }
    }

    /**
     * 放在Activity的 onPause()方法里
     */
    fun onPause() {
        builder?.run {
            mapView?.onPause()
        }
    }

    fun onSaveInstanceState(outState: Bundle?) {
        builder?.run {
            savedInstanceState = outState
        }
    }

    /**
     * 在build()之后马上执行
     */
    fun start(): MapDominator{
        builder?.run { initMap() }
        return this
    }

    /**
     * 设置定位按钮监听，点一下就重新执行定位。
     */
    fun setLocateButton(btn: View): MapDominator {
        btn.setOnClickListener { builder?.mlocationClient?.startLocation() }
        return this
    }

    /**
     * 激活
     */
    fun activateMap(
        listener: LocationSource.OnLocationChangedListener?,
        successCallback: (location: AMapLocation) -> Unit,
        failedCallback: (location: AMapLocation) -> Unit
    ) {
        builder?.run {
            mListener = listener
            if (mlocationClient == null && locationSource is Context) {
                val context = locationSource as Context
                AMapLocationClient.updatePrivacyAgree(context,true)
                AMapLocationClient.updatePrivacyShow(context,true,true)
                mlocationClient = AMapLocationClient(context).apply {
                    //设置为单次定位（区别于连续定位）
                    setLocationOption(AMapLocationClientOption().setOnceLocation(true))
                    setLocationListener {
                        if (mListener != null) {
                            if (it != null && it.errorCode == 0) {
                                //定位成功回调信息，设置相关消息
                                //取出经纬度（Latitude纬度；Longitude经度）
                                val latLng = LatLng(it.getLatitude(), it.getLongitude())
                                //然后可以移动到定位点,使用animateCamera就有动画效果
                                aMap?.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, 17f))
                                mListener?.onLocationChanged(it)
                                successCallback.invoke(it)
                            } else {
                                failedCallback.invoke(it)
                                Log.e(
                                    "AmapErr",
                                    "高德地图获取地理信息失败：${it.getErrorCode()} : ${it.getErrorInfo()}"
                                )
                            }
                        }
                    }
                    startLocation()
                }
            }
        }

    }

    //==========================builder=======================
    class MapBuilder(
        var savedInstanceState: Bundle?,
        val locationSource: LocationSource,
        val mapView: MapView?
    ) {
        var aMap: AMap? = mapView?.map
        var mListener: LocationSource.OnLocationChangedListener? = null
        var mlocationClient: AMapLocationClient? = null

        fun initMap() {
            //配置高德地图
            aMap?.apply {
                // 设置定位监听
                setLocationSource(locationSource)
                uiSettings.isZoomControlsEnabled = false//不显示缩放的加减号控件
                uiSettings.isMyLocationButtonEnabled = false//设置默认定位按钮是否显示，非必需设置。
                uiSettings.logoPosition = AMapOptions.LOGO_POSITION_BOTTOM_RIGHT
                //地图的缩放级别一共分为 17 级，从 3 到 19。数字越大，展示的图面信息越精细。
                moveCamera(CameraUpdateFactory.zoomTo(17f))
                setMyLocationEnabled(true)// 设置为true表示启动显示定位蓝点，false表示隐藏定位蓝点并不进行定位，默认是false。
            }
            setMyLocationStyle(aMap)
            mapView?.onCreate(savedInstanceState)
        }


        private fun setMyLocationStyle(aMap: AMap?): MapBuilder {
            //初始化定位蓝点样式类myLocationStyle.myLocationType(MyLocationStyle.LOCATION_TYPE_LOCATION_ROTATE);
            // 连续定位、且将视角移动到地图中心点，定位点依照设备方向旋转，并且会跟随设备移动。（1秒1次定位）如果不设置myLocationType，默认也会执行此种模式。
            //设置定位蓝点的Style
            aMap?.setMyLocationStyle(MyLocationStyle().apply {
                //设置连续定位模式下的定位间隔，只在连续定位模式下生效，单次定位模式下不会生效。单位为毫秒。
//            interval(2000)
                showMyLocation(true)//是否显示定位蓝点。
                strokeColor(Color.parseColor("#3FFC3F"))//精度圆圈边框
                strokeWidth(10f)
                radiusFillColor(Color.TRANSPARENT)//精度圆圈填充色
                anchor(
                    0.5f,
                    1f
                )//蓝点图标的锚点【例：如需要将图标的左下方像素点与定位蓝点的经纬度关联在一起，通过如下方法传入（0.0,1.0）。图标左上点为像素原点。】
                myLocationIcon(BitmapDescriptorFactory.fromResource(R.mipmap.map_location_anchor))//自定义锚点。
            })
            return this
        }

        fun build(): MapDominator {
            return MapDominator(this)
        }
    }
}