package com.wgmf.trade.utils;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapShader;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.RectF;
import android.graphics.Shader;

import androidx.annotation.RawRes;

import android.text.TextUtils;
import android.widget.ImageView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.resource.drawable.GlideDrawable;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;
import com.squareup.picasso.Callback;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Transformation;
import com.urovo.wugumofang.R;

import java.io.File;

import jp.wasabeef.glide.transformations.CropCircleTransformation;

/**
 * Picasso图片加载工具类
 *
 * @author samy
 * @date 16/9/27
 * <p>
 * Picasso.with(context).load(R.drawable.landing_screen).into(imageView1);
 * Picasso.with(context).load("file:///android_asset/DvpvklR.png").into(imageView2);
 * Picasso.with(context).load(new File(...)).into(imageView3);
 * <p>
 * 参照 UniversalImageLoader
 * <p>
 * "http://site.com/image.png" // from Web
 * "file:///mnt/sdcard/image.png" // from SD card
 * "file:///mnt/sdcard/video.mp4" // from SD card (video thumbnail)
 * "content://media/external/images/media/13" // from content provider
 * "content://media/external/video/media/13" // from content provider (video thumbnail)
 * "assets://image.png" // from assets
 * "drawable://" + R.drawable.img // from drawables (non-9patch images)
 */

public class ImageLoader {
    Context context;

    public ImageLoader(Context context) {
        this.context = context;
    }

    /**
     * 加载圆角图片
     */
    public void loadRoundImg(File file, ImageView img, final int corner, int width, int height) {
        Picasso.with(context).load(file).resize(width, height)/*.placeholder(R.drawable.image_place_holder)*/.transform(new Transformation() {
            @Override
            public Bitmap transform(Bitmap source) {
                final Paint paint = new Paint();
                paint.setAntiAlias(true);
                paint.setShader(new BitmapShader(source, Shader.TileMode.CLAMP, Shader.TileMode.CLAMP));

                Bitmap output = Bitmap.createBitmap(source.getWidth(), source.getHeight(), Bitmap.Config.ARGB_8888);
                Canvas canvas = new Canvas(output);
                canvas.drawRoundRect(new RectF(0, 0, source.getWidth(), source.getHeight()), corner, corner, paint);
                if (source != output) {
                    source.recycle();
                }
                return output;
            }

            @Override
            public String key() {
                return "circle";
            }
        }).into(img);
    }

    /**
     * 加载网络圆角图片
     */
    public void loadRoundImgUrl(String url, ImageView img, final int corner, int width, int height) {
        Picasso.with(context).load(url).resize(width, height)/*.placeholder(R.drawable.image_place_holder)*/.transform(new Transformation() {
            @Override
            public Bitmap transform(Bitmap source) {
                final Paint paint = new Paint();
                paint.setAntiAlias(true);
                paint.setShader(new BitmapShader(source, Shader.TileMode.CLAMP, Shader.TileMode.CLAMP));

                Bitmap output = Bitmap.createBitmap(source.getWidth(), source.getHeight(), Bitmap.Config.ARGB_8888);
                Canvas canvas = new Canvas(output);
                canvas.drawRoundRect(new RectF(0, 0, source.getWidth(), source.getHeight()), corner, corner, paint);
                if (source != output) {
                    source.recycle();
                }
                return output;
            }

            @Override
            public String key() {
                return "circle";
            }
        }).into(img);
    }


    public static void loadImgUrl(Context context, String url, ImageView iv) {
        Picasso.with(context).load(url).into(iv);
    }

    /**
     * 加载圆形网络图片
     */
    public static void loadCircleImg(Context context,
                                     String iconUrl, ImageView ivIcon,
                                     int imgLoading, int imgError) {
        Glide.with(context)
                .load(iconUrl)
                .centerCrop()
                .dontAnimate()//防止设置placeholder导致第一次不显示网络图片,只显示默认图片的问题
                .error(imgError)
                .placeholder(imgLoading)
                //任何图片都能裁剪成圆形的,除了占位符
                .bitmapTransform(new CropCircleTransformation(context))
                .into(ivIcon);
    }

    /**
     * 加载网络图片
     */
    public static void loadImg(Context context,
                               String iconUrl, ImageView ivIcon,
                               int imgLoading, int imgError) {
        Glide.with(context)
                .load(iconUrl)
                .centerCrop()
                .dontAnimate()//防止设置placeholder导致第一次不显示网络图片,只显示默认图片的问题
                .error(imgError)
                .placeholder(imgLoading)
                .into(ivIcon);
    }

    public static void loadImg(Context context, Integer iconUrl, ImageView ivIcon) {
        Glide.with(context)
                .load(iconUrl)
                .centerCrop()
                .dontAnimate()//防止设置placeholder导致第一次不显示网络图片,只显示默认图片的问题
                .error(R.mipmap.img_default_face)
                .placeholder(R.mipmap.img_default_face)
                //任何图片都能裁剪成圆形的,除了占位符
                .bitmapTransform(new CropCircleTransformation(context))
                .into(ivIcon);
    }

    public static void loadFileImg(Context context, File file, ImageView ivIcon, OnImageLoadingListener<File> listener) {
        Glide.with(context)
                .load(file)
                .listener(new RequestListener<File, GlideDrawable>() {
                    @Override
                    public boolean onException(Exception e, File file, Target<GlideDrawable> target, boolean b) {
                        listener.onException(e);
                        return false;
                    }

                    @Override
                    public boolean onResourceReady(GlideDrawable glideDrawable, File file, Target<GlideDrawable> target, boolean b, boolean b1) {
                        listener.onResourceReady(file);
                        return false;
                    }
                })
                .into(ivIcon);

    }

    public interface OnImageLoadingListener<T> {
        void onException(Exception e);

        void onResourceReady(T t);
    }

    public static void loadImg(Context context, String url, ImageView iv) {
        Picasso.with(context).load(TextUtils.isEmpty(url) ? null : url).into(iv);
    }

    public static void loadImgRes(Context context, String iconUrl, ImageView ivIcon) {
//        Picasso.with(context).load(TextUtils.isEmpty(url) ? null : url).placeholder(R.mipmap.img_default_face).placeholder(R.mipmap.img_default_face).into(iv);
        Glide.with(context)
                .load(iconUrl)
                .centerCrop()
                .dontAnimate()//防止设置placeholder导致第一次不显示网络图片,只显示默认图片的问题
                .error(R.drawable.default_head)
                .placeholder(R.drawable.default_head)
                //任何图片都能裁剪成圆形的,除了占位符
//                .bitmapTransform(new CropCircleTransformation(context))
                .into(ivIcon);
    }

    public static void loadImgRes(Context context, String url,
                                  ImageView iv, @RawRes int imgLoading, @RawRes int imgError) {
        Picasso.with(context).load(url).placeholder(imgLoading).placeholder(imgError).into(iv);
    }

    public static void loadImgCallback(Context context, String url, ImageView iv, Callback callback) {
        Picasso.with(context).load(url).into(iv, callback);
    }

    /**
     * 从内存卡中异步加载本地图片
     */
    public void displayFromSDCard(String uri, ImageView imageView) {
        // String imageUri = "file:///mnt/sdcard/image.png"; // from SD card
        //ImageLoader.getInstance().displayImage("file://" + uri, imageView);
    }


    /**
     * 从assets文件夹中异步加载图片
     * <p>
     * //   * @param imageName 图片名称，带后缀的，例如：1.png
     */
    public static void dispalyFromAssets(Context context, String url, ImageView iv) {
        // String imageUri = "assets://image.png"; // from assets
        //ImageLoader.getInstance().displayImage("assets://" + imageName, imageView);
        Picasso.with(context).load("file:///android_asset/" + url).into(iv);
    }

    /**
     * 从drawable中异步加载本地图片
     */
    public void displayFromDrawable(int imageId, ImageView imageView) {
        // String imageUri = "drawable://" + R.drawable.image; // from drawables
        // (only images, non-9patch)
        //ImageLoader.getInstance().displayImage("drawable://" + imageId, imageView);
    }

    /**
     * 从内容提提供者中抓取图片
     */
    public void displayFromContent(String uri, ImageView imageView) {
        // String imageUri = "content://media/external/audio/albumart/13"; //
        // from content provider
        //ImageLoader.getInstance().displayImage("content://" + uri, imageView);
    }
}
